#!/usr/bin/bash

# endpoints=192.168.1.5
# interface=wlp0s20f3
# filename="/home/llh/ros-kubernetes/docker/pinger/pingStats.json"
# gpstopic="/dji_osdk_ros/gps_position"

echo "pinging: $endpoints"
echo "through: $interface"
echo "save to: $filename"
echo "gps topic: $gpstopic"
echo " "

if test -f "$filename"; then
  echo "file $filename already exists."
else
  touch "$filename"
  echo "{}" > "$filename"
fi

# enable ros commands
source /ros_entrypoint.sh

echo ping "${endpoints} every ${downtime:=3} sec, saving to ${filename}"
# for endpoints in "${endpoints[@]}"; do
# done
while true; do

  # for endpoints in "${endpoints[@]}"; do

  # echo pinging "${endpoints} now ..."
  newData=$(ping -c 3 ${endpoints:=localhost} | tail -n 1)

  # if nothing comes back, we dont write anything
  if [[ -n "$newData" ]]; then
    false
    echo "$newData"
  else
    echo "${endpoints} might be unreachable"
    continue
  fi

  OLDIFS=$IFS
  IFS=" "
  read bit1 bit2 bit3 bit4 bit5 <<< "$newData"
  IFS="/"
  read num1 num2 num3 num4 <<< "$bit4"
  IFS=$OLDIFS

  if test -f "$filename"; then
    false
  else
    touch "$filename"
    echo "{}" > "$filename"
  fi

  # get gps data if present (timeout 5s)
  gpsData=$(timeout 5 rostopic echo -n 1 $gpstopic | grep "latitude:\|longitude:\|altitude:")
  gpsData=$(echo $gpsData | tr '\n' ' ')
  OLDIFS=$IFS
  IFS=" "
  read trash1 latitude trash2 longitude trash3 altitude <<< "$gpsData"
  IFS=$OLDIFS

  echo $latitude
  echo $longitude
  echo $altitude
  echo "------------------"

  # ping can return standard deviation and gps data may not be available
  if [[ -z "$latitude" ]]; then
    # echo "gps data is not there"
    if [[ -n "$num4" ]]; then
      jq --arg timedate $(date +"%H:%M_%F") --arg target $endpoints --argjson rttMin $num1 --argjson rttAvg $num2 --argjson rttMax $num3 --argjson stdDev $num4 '.[$target][$timedate] += [{"min":$rttMin,"avg":$rttAvg,"max":$rttMax,"std":$stdDev}]' "$filename" > "$filename.tmp" && mv "$filename.tmp" "$filename"
    else
      jq --arg timedate $(date +"%H:%M_%F") --arg target $endpoints --argjson rttMin $num1 --argjson rttAvg $num2 --argjson rttMax $num3 '.[$target][$timedate] += [{"min":$rttMin,"avg":$rttAvg,"max":$rttMax}]' "$filename" > "$filename.tmp" && mv "$filename.tmp" "$filename"
    fi
  else
    echo "gps data IS there"
    echo $latitude
    echo $longitude
    echo $altitude
    if [[ -n "$num4" ]]; then
      jq --arg timedate $(date +"%H:%M_%F") --arg target $endpoints --argjson altitude $altitude --argjson longitude $longitude --argjson latitude $latitude --argjson rttMin $num1 --argjson rttAvg $num2 --argjson rttMax $num3 --argjson stdDev $num4 '.[$target][$timedate] += [{"lat":$latitude,"long":$longitude,"alt":$altitude,"min":$rttMin,"avg":$rttAvg,"max":$rttMax,"std":$stdDev}]' "$filename" > "$filename.tmp" && mv "$filename.tmp" "$filename"
    else
      jq --arg timedate $(date +"%H:%M_%F") --arg target $endpoints --argjson altitude $altitude --argjson longitude $longitude --argjson latitude $latitude --argjson rttMin $num1 --argjson rttAvg $num2 --argjson rttMax $num3 '.[$target][$timedate] += [{"lat":$latitude,"long":$longitude,"alt":$altitude,"min":$rttMin,"avg":$rttAvg,"max":$rttMax}]' "$filename" > "$filename.tmp" && mv "$filename.tmp" "$filename"
    fi
  fi
  sleep ${downtime:=3}
done

if test -f "$filename.tmp"; then
  rm "$filename.tmp"
fi

# if test -f "$nfsFilename.tmp"; then
#   rm "$nfsFilename.tmp"
# fi
